"use strict";
/**
 * @file Fonctions qui implémentent les requetes relatives aux utilisateurs
 * @author ofacklam
 * @memberof GraphQL
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const users_1 = require("../object_resolvers/users");
const user_1 = require("../../ldap/export/user");
const apollo_server_core_1 = require("apollo-server-core");
class UserModel {
    /**
     * @memberof GraphQL
     * @class UserModel
     * @summary Requetes relatives aux utilisateurs.
     * @classdesc Cette classe contient les méthodes implémentant les requetes relatives aux utilisateurs.
     * @arg {string} contextUser - L'identifiant de l'utilisateur du 'context'
     */
    constructor(contextUser) {
        this.contextUser = contextUser;
    }
    /**
     * @memberof GraphQL.UserModel#
     * @function getUser
     * @summary Fonction qui renvoit un utilisateur donné.
     * @arg {string} uid - Identifiant demandé.
     * @return {Promise(User)} Renvoie l'utilisateur dont l'identifiant est 'uid'
     * @async
     * @rights connectedOrOnplatal
     */
    getUser(uid) {
        return __awaiter(this, void 0, void 0, function* () {
            return users_1.User.tryCreate(uid);
        });
    }
    /**
     * @memberof GraphQL.UserModel#
     * @function searchTOL
     * @summary Fonction qui recherche dans le TOL
     * @arg {searchTOLArgs} args - les données de recherche
     * @return {Promise(User[])} Renvoie une liste d'utilisateurs
     * @async
     * @rights connectedOrOnplatal
     */
    searchTOL(args) {
        return __awaiter(this, void 0, void 0, function* () {
            //TODO : correctly handle groups (in LDAP, a member can be stored as member, speaker or admin...)
            const searchData = new user_1.userData();
            searchData.givenName = args.givenName;
            searchData.lastName = args.lastName;
            searchData.nickname = args.nickname;
            searchData.nationality = args.nationality;
            searchData.members = args.groups;
            searchData.phone = args.phone;
            searchData.mail = args.mail;
            searchData.address = args.address;
            const userList = yield user_1.User.search(searchData);
            return userList.map((uid) => new users_1.User(uid));
        });
    }
    /**
     * @memberof GraphQL.UserModel#
     * @function editProfile
     * @summary Fonction qui modifie le profil et renvoie l'utilisateur
     * @arg {editProfileArgs} args - les données a modifier
     * @return {Promise(User)} Renvoie l'utilisateur mis a jour
     * @async
     * @rights authenticated
     */
    editProfile(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let data = yield user_1.User.peek(this.contextUser);
            //Modify some fields, keep the others
            let editArgs = new user_1.userData();
            editArgs.uid = data.uid;
            editArgs.password = data.password;
            editArgs.givenName = data.givenName;
            editArgs.lastName = data.lastName;
            editArgs.nickname = args.nickname; // <- this field is modified by user
            editArgs.gender = data.gender;
            editArgs.photo = data.photo;
            editArgs.phone = args.phone; // <- this field is modified
            editArgs.address = data.address; // WTF why can't this be changed ????
            editArgs.mail = args.mail; // <- this field is modified
            editArgs.birthdate = data.birthdate;
            editArgs.nationality = data.nationality;
            editArgs.admins = data.admins;
            editArgs.speakers = data.speakers;
            editArgs.members = data.members;
            editArgs.followers = data.followers;
            if (yield user_1.User.edit(editArgs)) {
                return new users_1.User(data.uid);
            }
            else {
                throw new apollo_server_core_1.ApolloError("Edit failed");
            }
        });
    }
}
exports.UserModel = UserModel;
//# sourceMappingURL=userModel.js.map