"use strict";
/**
 * @file Resolvers des groupes
 * @author ofacklam
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const users_1 = require("./users");
const group_1 = require("../../ldap/export/group");
const knex_router_1 = __importDefault(require("../../../db/knex_router"));
const apollo_server_core_1 = require("apollo-server-core");
class Group {
    /**
     * @abstract
     * @memberof GraphQL
     * @class Group
     * @summary Resolvers des groupes
     * @classdesc Une classe abstraite représentant l'interface group du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {string} gid - Identifiant du groupe, supposé valide
     * @rights connectedOrOnplatal
     */
    constructor(gid) {
        this.gid = gid;
        this.m_dataLoaded = false;
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.Group#
     * @function __resolveType
     * @summary Renvoie si c'est un SimpleGroup ou un MetaGroup
     * @return {string}
     * @rights connectedOrOnplatal
     */
    __resolveType(args, context, info) {
        if (this instanceof SimpleGroup) {
            return "SimpleGroup";
        }
        else if (this instanceof MetaGroup) {
            return "MetaGroup";
        }
        else {
            throw new apollo_server_core_1.ApolloError("Bad group type");
        }
    }
    /**
     * @memberof GraphQL.Group#
     * @function createdAt
     * @summary Renvoie la date de création
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    createdAt(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_createdAt;
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function updatedAt
     * @summary Renvoie la date de mise a jour
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    updatedAt(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_updatedAt;
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function name
     * @summary Renvoie le nom du groupe
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    name(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_name;
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function description
     * @summary Renvoie la description du groupe
     * @return {Promise(string)}
     * @rights viewer
     * @async
     */
    description(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                yield this.fetchData();
                return this.m_description;
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function mail
     * @summary Renvoie l'adresse mail
     * @return {Promise(string)}
     * @rights viewer
     * @async
     */
    mail(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                yield this.fetchData();
                return this.m_mail;
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function website
     * @summary Renvoie l'adresse du site web
     * @return {Promise(string)}
     * @rights viewer
     * @async
     */
    website(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                yield this.fetchData();
                return this.m_website;
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function frontPage
     * @summary Renvoie la page principale
     * @return {Promise(string)}
     * @rights viewer
     * @async
     */
    frontPage(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                yield this.fetchData();
                return this.m_frontPage;
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function questions
     * @summary Renvoie les questions adressées a ce groupe
     * @return {Promise(Question[])}
     * @rights viewer
     * @async
     */
    questions(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                throw "Not implemented";
                // let result = await knex('questions').select().whereIn('id', received_messages);
                // for(let entry of result){
                //     entry.type = "Question";
                // }
                // return result;
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function answers
     * @summary Renvoie les réponses de ce groupe
     * @return {Promise(Answer[])}
     * @rights viewer
     * @async
     */
    answers(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                throw "Not implemented";
                // let received_messages = await selectors.recievedMessages(user, groupUID);
                // let result = await knex('answers').select().whereIn('id', received_messages);
                // for(let entry of result){
                //     entry.type = "Answer";
                // }
                // return result;
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function announcementsFromGroup
     * @summary Renvoie les annonces créées par ce groupe
     * @return {Promise(Announcement[])}
     * @rights member
     * @async
     */
    announcementsFromGroup(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isMember(this.gid)) {
                throw "Not implemented";
            }
            throw new apollo_server_core_1.AuthenticationError("Not a member");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function announcementsToGroup
     * @summary Renvoie les annonces adressées a ce groupe
     * @return {Promise(Announcement[])}
     * @rights member
     * @async
     */
    announcementsToGroup(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isMember(this.gid)) {
                throw "Not implemented";
            }
            throw new apollo_server_core_1.AuthenticationError("Not a member");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function eventsFromGroup
     * @summary Renvoie les evenements créés par ce groupe
     * @return {Promise(Event[])}
     * @rights member
     * @async
     */
    eventsFromGroup(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isMember(this.gid)) {
                throw "Not implemented";
            }
            throw new apollo_server_core_1.AuthenticationError("Not a member");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function eventsToGroup
     * @summary Renvoie les evenements adressées a ce groupe
     * @return {Promise(Event[])}
     * @rights member
     * @async
     */
    eventsToGroup(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isMember(this.gid)) {
                throw "Not implemented";
            }
            throw new apollo_server_core_1.AuthenticationError("Not a member");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function privatePosts
     * @summary Renvoie les posts internes du groupe
     * @return {Promise(PrivatePost[])}
     * @rights member
     * @async
     */
    privatePosts(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isMember(this.gid)) {
                throw "Not implemented";
            }
            throw new apollo_server_core_1.AuthenticationError("Not a member");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function postsSummary
     * @summary Renvoie le résumé interne
     * @return {Promise(string)}
     * @rights member
     * @async
     */
    postsSummary(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isMember(this.gid)) {
                yield this.fetchData();
                return this.m_postsSummary;
            }
            throw new apollo_server_core_1.AuthenticationError("Not a member");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function requestsToGroup
     * @summary Renvoie toutes les requetes adressées au groupe.
     * @return {Promise(Request[])}
     * @rights admin
     * @async
     */
    requestsToGroup(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isAdmin(this.gid)) {
                return context.models.request.getRequestsToGroup(this.gid);
            }
            throw new apollo_server_core_1.AuthenticationError("Not an admin");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function userJoinGroupRequestsToGroup
     * @summary Renvoie toutes les requetes UserJoinGroup adressées au groupe.
     * @return {Promise(UserJoinGroup[])}
     * @rights admin
     * @async
     */
    userJoinGroupRequestsToGroup(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isAdmin(this.gid)) {
                return context.models.request.getUserJoinGroupRequestsToGroup(this.gid);
            }
            throw new apollo_server_core_1.AuthenticationError("Not an admin");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function groupJoinMetagroupRequestsToGroup
     * @summary Renvoie toutes les requetes GroupJoinMetagroup adressées au groupe.
     * @return {Promise(GroupJoinMetagroup[])}
     * @rights admin
     * @async
     */
    groupJoinMetagroupRequestsToGroup(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isAdmin(this.gid)) {
                return context.models.request.getGroupJoinMetagroupRequestsToGroup(this.gid);
            }
            throw new apollo_server_core_1.AuthenticationError("Not an admin");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function groupCoauthorEventRequestsToGroup
     * @summary Renvoie toutes les requetes GroupCoauthorEvent adressées au groupe.
     * @return {Promise(GroupCoauthorEvent[])}
     * @rights admin
     * @async
     */
    groupCoauthorEventRequestsToGroup(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isAdmin(this.gid)) {
                return context.models.request.getGroupCoauthorEventRequestsToGroup(this.gid);
            }
            throw new apollo_server_core_1.AuthenticationError("Not an admin");
        });
    }
    /**
     * @memberof GraphQL.Group#
     * @function visibilityEdges
     * @summary Renvoie les groupes auxquels ce groupe est visible.
     * @return {Promise(Group[])}
     * @rights viewer
     * @async
     */
    visibilityEdges(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                throw "Not implemented";
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
}
exports.Group = Group;
class SimpleGroup extends Group {
    /**
     * @memberof GraphQL
     * @class SimpleGroup
     * @extends GraphQL.Group
     * @summary Resolvers des groupes simples
     * @classdesc Une classe représentant le type SimpleGroup du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {string} gid - Identifiant du groupe simple, supposé valide.
     * @rights connectedOrOnplatal
     */
    constructor(gid) {
        super(gid);
    }
    /**
     * @memberof GraphQL.SimpleGroup#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer le groupe simple correspondant
     * @arg {string} gid - Identifiant du groupe simple, sans hypothese sur la validité.
     * @returns {Promise(SimpleGroup)} - Renvoie le groupe simple créé, ou null en cas d'erreur.
     * @rights connectedOrOnplatal
     * @async
     * @static
     */
    static tryCreate(gid) {
        return __awaiter(this, void 0, void 0, function* () {
            let g = new SimpleGroup(gid);
            if (yield g.fetchData()) {
                return g;
            }
            return null;
        });
    }
    /**
     * @memberof GraphQL.SimpleGroup#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce groupe simple dans le LDAP, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.m_dataLoaded) {
                try {
                    let data = yield group_1.Group.peek(this.gid);
                    if (data.gid !== this.gid) {
                        throw "Error in search";
                    }
                    //this.m_createdAt = data.createdAt;
                    //this.m_updatedAt = data.updatedAt;
                    this.m_name = data.name;
                    this.m_description = data.description;
                    //this.m_mail = data.mail;
                    //this.m_website = data.website;
                    this.m_members = data.members;
                    //this.m_speakers = data.speakers;
                    this.m_admins = data.admins;
                    //this.m_likers = data.likers;
                    //this.m_frontPage = data.frontPage;
                    //this.m_postsSummary = data.postsSummary;
                    //this.m_parent = data.parent;
                    //this.m_children = data.children;
                    //this.m_school = ???;
                    this.m_dataLoaded = true;
                    return true;
                }
                catch (_a) {
                    return false;
                }
            }
            return true;
        });
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.SimpleGroup#
     * @function members
     * @summary Renvoie la liste des membres
     * @return {Promise(User[])}
     * @rights viewer
     * @async
     */
    members(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                yield this.fetchData();
                return this.m_members.map(uid => {
                    return new users_1.User(uid);
                });
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.SimpleGroup#
     * @function speakers
     * @summary Renvoie la liste des speakers
     * @return {Promise(User[])}
     * @rights viewer
     * @async
     */
    speakers(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                yield this.fetchData();
                return this.m_speakers.map(uid => {
                    return new users_1.User(uid);
                });
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.SimpleGroup#
     * @function admins
     * @summary Renvoie la liste des admins
     * @return {Promise(User[])}
     * @rights viewer
     * @async
     */
    admins(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                yield this.fetchData();
                return this.m_admins.map(uid => {
                    return new users_1.User(uid);
                });
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.SimpleGroup#
     * @function likers
     * @summary Renvoie la liste des sympathisants
     * @return {Promise(User[])}
     * @rights viewer
     * @async
     */
    likers(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                yield this.fetchData();
                return this.m_likers.map(uid => {
                    return new users_1.User(uid);
                });
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.SimpleGroup#
     * @function parent
     * @summary Renvoie le groupe parent
     * @return {Promise(SimpleGroup)}
     * @rights viewer
     * @async
     */
    parent(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                yield this.fetchData();
                if (this.m_parent == 'root') {
                    return null;
                }
                else {
                    return new SimpleGroup(this.m_parent);
                }
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.SimpleGroup#
     * @function children
     * @summary Renvoie la liste des groupes enfants
     * @return {Promise(SimpleGroup[])}
     * @rights viewer
     * @async
     */
    children(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                yield this.fetchData();
                return this.m_children.map(gid => {
                    return new SimpleGroup(gid);
                });
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.SimpleGroup#
     * @function memberOfMeta
     * @summary Renvoie la liste des méta-groupes dont ce groupe fait partie
     * @return {Promise(MetaGroup[])}
     * @rights viewer
     * @async
     */
    memberOfMeta(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                throw "Not implemented";
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.SimpleGroup#
     * @function school
     * @summary Renvoie l'école d'origine
     * @return {Promise(string)}
     * @rights viewer
     * @async
     */
    school(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                yield this.fetchData();
                return this.m_school;
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
}
exports.SimpleGroup = SimpleGroup;
class MetaGroup extends Group {
    /**
     * @memberof GraphQL
     * @class MetaGroup
     * @extends GraphQL.Group
     * @summary Resolvers des méta-groupes
     * @classdesc Une classe représentant le type MetaGroup du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {string} gid - Identifiant du méta-groupe, supposé valide
     * @rights connectedOrOnplatal
     */
    constructor(gid) {
        super(gid);
    }
    /**
     * @memberof GraphQL.MetaGroup#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer le méta-groupe correspondant
     * @arg {string} gid - Identifiant du méta-groupe, sans hypothese sur la validité.
     * @returns {Promise(MetaGroup)} - Renvoie le méta-groupe créé, ou null en cas d'erreur.
     * @rights connectedOrOnplatal
     * @async
     * @static
     */
    static tryCreate(gid) {
        return __awaiter(this, void 0, void 0, function* () {
            let g = new MetaGroup(gid);
            if (yield g.fetchData()) {
                return g;
            }
            return null;
        });
    }
    /**
     * @memberof GraphQL.MetaGroup#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce méta-groupe dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.m_dataLoaded) {
                let data = yield knex_router_1.default.select('created_at', 'updated_at', 'name', 'description', 
                //'mail', 
                //'posts_summary', 
                //'front_page', 
                'website').from('groups').where('gid', this.gid);
                if (data.length > 0) {
                    let g = data[0];
                    this.m_createdAt = g.created_at;
                    this.m_updatedAt = g.updated_at;
                    this.m_name = g.name;
                    this.m_description = g.description;
                    //this.m_mail = g.mail;
                    this.m_website = g.website;
                    //this.m_frontPage = g.frontPage;
                    //this.m_postsSummary = g.postsSummary;
                    this.m_dataLoaded = true;
                    return true;
                }
                else {
                    return false;
                }
            }
            return true;
        });
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.MetaGroup#
     * @function admins
     * @summary Renvoie la liste des admins
     * @return {Promise(User[])}
     * @rights viewer
     * @async
     */
    admins(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                throw "Not implemented";
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
    /**
     * @memberof GraphQL.MetaGroup#
     * @function members
     * @summary Renvoie la liste des groupes membres
     * @return {Promise(SimpleGroup[])}
     * @rights viewer
     * @async
     */
    members(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            if (context.models.auth.isViewer(this.gid)) {
                throw "Not implemented";
                /*let member_group_list = await knex.distinct().select().from('groups')
                    .innerJoin('meta_group_membership', 'groups.uid', 'meta_group_membership.member_uid')
                    .where('meta_group_membership.union_uid', '=', group.gid);
                let members = new Set;
                for (const memberGroup of member_group_list) {
                    let res = await getGroupMemberUsers(quser, new Group(memberGroup));
                    for (const member of res.values()) {
                        members.add(member);
                    }
                }
                return members;*/
            }
            throw new apollo_server_core_1.AuthenticationError("Not a viewer");
        });
    }
}
exports.MetaGroup = MetaGroup;
//# sourceMappingURL=groups.js.map