"use strict";
/**
 * @file Resolvers pour tous les types de messages
 * @author akka, ofacklam
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const groups_1 = require("./groups");
const users_1 = require("./users");
const messages_1 = require("./messages");
const knex_router_1 = __importDefault(require("../../../db/knex_router"));
const apollo_server_core_1 = require("apollo-server-core");
class Request {
    /**
     * @memberof GraphQL
     * @class Request
     * @summary Resolvers des requetes
     * @classdesc Une classe abstraite représentant l'interface Request du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} rid - Identifiant de la requete, supposé valide.
     * @rights dépend du type de requete
     * @abstract
     */
    constructor(rid) {
        this.rid = rid;
        this.m_dataLoaded = false;
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.Request#
     * @function __resolveType
     * @summary Renvoie quel type de Request c'est
     * @return {string}
     * @rights same as request object
     */
    __resolveType() {
        if (this instanceof UserJoinGroup) {
            return "UserJoinGroup";
        }
        else if (this instanceof GroupJoinMetagroup) {
            return "GroupJoinMetagroup";
        }
        else if (this instanceof GroupCoauthorEvent) {
            return "GroupCoauthorEvent";
        }
        else {
            throw new apollo_server_core_1.ApolloError("Bad request type");
        }
    }
    /**
     * @memberof GraphQL.Request#
     * @function comment
     * @summary Renvoie le commentaire associé
     * @return {Promise(string)}
     * @rights same as request object
     * @async
     */
    comment(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return this.m_comment;
        });
    }
}
exports.Request = Request;
class UserJoinGroup extends Request {
    /**
     * @memberof GraphQL
     * @class UserJoinGroup
     * @extends GraphQL.Request
     * @summary Resolvers des requetes UserJoinGroup
     * @classdesc Une classe représentant le type UserJoinGroup du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} rid - Identifiant de la requete, supposé valide.
     * @rights admin du groupe recipient ou le user émetteur
     */
    constructor(rid) {
        super(rid);
    }
    /**
     * @memberof GraphQL.UserJoinGroup#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer la requete correspondante
     * @arg {number} rid - Identifiant de la requete, sans hypothese sur la validité.
     * @returns {Promise(UserJoinGroup)} - Renvoie le UserJoinGroup créé, ou null en cas d'erreur.
     * @rights admin du groupe recipient ou le user émetteur
     * @async
     * @static
     */
    static tryCreate(rid) {
        return __awaiter(this, void 0, void 0, function* () {
            let r = new UserJoinGroup(rid);
            if (yield r.fetchData()) {
                return r;
            }
            return null;
        });
    }
    /**
     * @memberof GraphQL.UserJoinGroup#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur cette requete dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.m_dataLoaded) {
                let data = yield knex_router_1.default.select('request_comment', 'request_from', 'request_to').from('requests_user_join_group').where('rid', this.rid);
                if (data.length > 0) {
                    let r = data[0];
                    this.m_comment = r.request_comment;
                    this.m_from = r.request_from;
                    this.m_to = r.request_to;
                    this.m_dataLoaded = true;
                    return true;
                }
                else {
                    return false;
                }
            }
            return true;
        });
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.UserJoinGroup#
     * @function from
     * @summary Renvoie le user émetteur
     * @return {Promise(User)}
     * @rights admin du groupe recipient ou le user émetteur
     * @async
     */
    from(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return new users_1.User(this.m_from);
        });
    }
    /**
     * @memberof GraphQL.UserJoinGroup#
     * @function to
     * @summary Renvoie le groupe destinataire
     * @return {Promise(SimpleGroup)}
     * @rights admin du groupe recipient ou le user émetteur
     * @async
     */
    to(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return new groups_1.SimpleGroup(this.m_to);
        });
    }
}
exports.UserJoinGroup = UserJoinGroup;
class GroupJoinMetagroup extends Request {
    /**
     * @memberof GraphQL
     * @class GroupJoinMetagroup
     * @extends GraphQL.Request
     * @summary Resolvers des requetes GroupJoinMetagroup
     * @classdesc Une classe représentant le type GroupJoinMetagroup du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} rid - Identifiant de la requete, supposé valide.
     * @rights admin du groupe émetteur ou destinataire
     */
    constructor(rid) {
        super(rid);
    }
    /**
     * @memberof GraphQL.GroupJoinMetagroup#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer la requete correspondante
     * @arg {number} rid - Identifiant de la requete, sans hypothese sur la validité.
     * @returns {Promise(GroupJoinMetagroup)} - Renvoie le GroupJoinMetagroup créé, ou null en cas d'erreur.
     * @rights admin du groupe émetteur ou destinataire
     * @async
     * @static
     */
    static tryCreate(rid) {
        return __awaiter(this, void 0, void 0, function* () {
            let r = new GroupJoinMetagroup(rid);
            if (yield r.fetchData()) {
                return r;
            }
            return null;
        });
    }
    /**
     * @memberof GraphQL.GroupJoinMetagroup#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur cette requete dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.m_dataLoaded) {
                let data = yield knex_router_1.default.select('request_comment', 'request_from', 'request_to').from('requests_group_join_metagroup').where('rid', this.rid);
                if (data.length > 0) {
                    let r = data[0];
                    this.m_comment = r.request_comment;
                    this.m_from = r.request_from;
                    this.m_to = r.request_to;
                    this.m_dataLoaded = true;
                    return true;
                }
                else {
                    return false;
                }
            }
            return true;
        });
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.GroupJoinMetagroup#
     * @function from
     * @summary Renvoie le groupe émetteur
     * @return {Promise(SimpleGroup)}
     * @rights admin du groupe émetteur ou destinataire
     * @async
     */
    from(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return new groups_1.SimpleGroup(this.m_from);
        });
    }
    /**
     * @memberof GraphQL.GroupJoinMetagroup#
     * @function to
     * @summary Renvoie le groupe destinataire
     * @return {Promise(MetaGroup)}
     * @rights admin du groupe émetteur ou destinataire
     * @async
     */
    to(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            return new groups_1.MetaGroup(this.m_to);
        });
    }
}
exports.GroupJoinMetagroup = GroupJoinMetagroup;
class GroupCoauthorEvent extends Request {
    /**
     * @memberof GraphQL
     * @class GroupCoauthorEvent
     * @extends GraphQL.Request
     * @summary Resolvers des requetes GroupCoauthorEvent
     * @classdesc Une classe représentant le type GroupCoauthorEvent du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} rid - Identifiant de la requete, supposé valide.
     * @rights admin du groupe émetteur ou destinataire
     */
    constructor(rid) {
        super(rid);
    }
    /**
     * @memberof GraphQL.GroupCoauthorEvent#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer la requete correspondante
     * @arg {number} rid - Identifiant de la requete, sans hypothese sur la validité.
     * @returns {Promise(GroupCoauthorEvent)} - Renvoie le GroupCoauthorEvent créé, ou null en cas d'erreur.
     * @rights admin du groupe émetteur ou destinataire
     * @async
     * @static
     */
    static tryCreate(rid) {
        return __awaiter(this, void 0, void 0, function* () {
            let r = new GroupCoauthorEvent(rid);
            if (yield r.fetchData()) {
                return r;
            }
            return null;
        });
    }
    /**
     * @memberof GraphQL.GroupCoauthorEvent#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur cette requete dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    fetchData() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.m_dataLoaded) {
                let data = yield knex_router_1.default.select('request_comment', 'request_from', 'request_to', 'for_event').from('requests_group_coauthor_event').where('rid', this.rid);
                if (data.length > 0) {
                    let r = data[0];
                    this.m_comment = r.request_comment;
                    this.m_from = r.request_from;
                    this.m_to = r.request_to;
                    this.m_forEvent = r.for_event;
                    this.m_dataLoaded = true;
                    return true;
                }
                else {
                    return false;
                }
            }
            return true;
        });
    }
    /**
     * Ci-dessous les resolvers a proprement parler.
     */
    /**
     * @memberof GraphQL.GroupCoauthorEvent#
     * @function from
     * @summary Renvoie le groupe émetteur
     * @return {Promise(Group)}
     * @rights admin du groupe émetteur ou destinataire
     * @async
     */
    from(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupCoauthorEvent#
     * @function to
     * @summary Renvoie le groupe destinataire
     * @return {Promise(Group)}
     * @rights admin du groupe émetteur ou destinataire
     * @async
     */
    to(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            throw "Not implemented";
        });
    }
    /**
     * @memberof GraphQL.GroupCoauthorEvent#
     * @function forEvent
     * @summary Renvoie l'evenement a co-organiser
     * @return {Promise(Event)}
     * @rights admin du groupe émetteur ou destinataire
     * @async
     */
    forEvent(args, context, info) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fetchData();
            throw "Not implemented";
            //a-t-il le droit de voir ???
            return new messages_1.Event(this.m_forEvent);
        });
    }
}
exports.GroupCoauthorEvent = GroupCoauthorEvent;
//# sourceMappingURL=requests.js.map